
#include "Arduino.h"

#define ENABLE 10
#define TE_MESURES 2000 // microsecondes
#define N_MESURES 500
#define AREF 4.97
#define VZERO_0 2.5
#define G_1 6.25e-3 // tension
#define G_0 1.62 //courant
#define A 6.5
#define TEREF 12.2
#define F_MOINS 2 // PE4
#define F_PLUS 3 // PE5

uint32_t icr;
float period_pwm,period_pwm_ref;
uint16_t diviseur[6] = {0,1,8,64,256,1024};
volatile uint32_t inter2;

float somme_0, somme_1, somme_2, somme_3;
char reglage,ajust;
float vzero_2;
float dephasage;
float Ueff;
float Ieff;
float Temp;
float temps;
char f_plus,f_moins;
                 
void init_pwm_timer1_fixe(float period, float ratio) { // rapport cyclique fixe, period microseoncondes
      char clockBits;
    TCCR1A = 0;
    TCCR1A |= (1 << COM1A1); //Clear OC1A on compare match when upcounting, set OC1A on compare match when downcounting
    TCCR1B = 1 << WGM13; // phase and frequency correct pwm mode, top = ICR1
    int d = 1;
    icr = (F_CPU/1000000*period/2);
    while ((icr>0xFFFF)&&(d<6)) { // choix du diviseur d'horloge
        d++;
        icr = (F_CPU/1000000*period/2/diviseur[d]);
   } 
   clockBits = d;
   ICR1 = icr; // valeur maximale du compteur
   OCR1A = ratio*icr;
   OCR1B = icr;
   TCNT1 = 0; // mise à zéro du compteur
   TCCR1B |= clockBits; // déclenchement du compteur
}
                 
void timer2_init(uint32_t period) { // compteur 8 bits 
  TCCR2A = 0;
  TCCR2B = 0;
  TCCR2B |= (1 << WGM21); // mode CTC avec OCR0A pour le maximum 
  uint32_t top = (F_CPU/1000000*period);
  int clock = 1;
  while ((top > 0xFF)&&(clock<5)) {
    clock ++;
    top = (F_CPU/1000000*period/diviseur[clock]);
  }
  OCR2A = top; // période
  TIMSK2 = (1 << OCIE2A); // interruption lorsque TCNT0 = OCR0A
  inter2 = 0;
  TCCR2B |= clock;
}

                 
void adc_init(uint8_t prescaler) {
  ADCSRA = 0;
  ADCSRA |= (1 << ADEN); 
  ADCSRA |= prescaler;
  
  ADCSRB = 0;  
}
                 
ISR(TIMER2_COMPA_vect) {
    ADMUX = 0b01000000 | 0; // voie 0 
    ADCSRA |= 0b01000000;
    while (ADCSRA & 0b01000000);
    float x0 = ((ADCL | (ADCH << 8))*AREF/1024);
    somme_0 += x0;
    ADMUX = 0b01000001 | 0; // voie 1 
    ADCSRA |= 0b01000000;
    while (ADCSRA & 0b01000000);
    float x1 = ((ADCL | (ADCH << 8))*AREF/1024);
    somme_1 += x1;
    ADMUX = 0b01000010 | 0; // voie 2
    ADCSRA |= 0b01000000;
    while (ADCSRA & 0b01000000);
    float x2 = ((ADCL | (ADCH << 8))*AREF/1024);
    somme_2 += x2;
    ADMUX = 0b01000011 | 0; // voie 3
    ADCSRA |= 0b01000000;
    while (ADCSRA & 0b01000000);
    float x3 = ((ADCL | (ADCH << 8)));
    somme_3 += x3;
    inter2++;
    
    if (inter2==N_MESURES) {
      inter2 = 0;
      dephasage = somme_2/N_MESURES-vzero_2;
      Ueff = sqrt((somme_1/N_MESURES)/G_1);
      Ieff = sqrt((somme_0/N_MESURES)/G_0)*A*period_pwm/TEREF;
      Temp = (somme_3/N_MESURES-155)*65.0/490+25.0;
      if (reglage) vzero_2 = somme_2/N_MESURES;
      somme_0 = somme_1 = somme_2 =  somme_3 = 0.0;
    }
#if defined(__AVR_ATmega2560__)
    if (PINE & (1<<PINE4)) {f_moins=1;f_plus=0;}
    if (PINE & (1<<PINE5)) {f_plus=1;f_moins=0;}
#else
   if (PIND & (1<<PIND2)) {f_moins=1;f_plus=0;}
   if (PIND & (1<<PIND3)) {f_plus=1;f_moins=0;}
#endif
}   
                   
void setup() {
  Serial.begin(115200);
  pinMode(ENABLE,OUTPUT);
  digitalWrite(ENABLE,LOW);
  
#if defined(__AVR_ATmega2560__)
    pinMode(11,OUTPUT); // sortie PWM pour MEGA
    digitalWrite(11,LOW);
#else
    pinMode(9,OUTPUT); // sortie PWM pour UNO
    digitalWrite(9,LOW);
#endif
    pinMode(F_MOINS,INPUT);
    pinMode(F_PLUS,INPUT);
    period_pwm = period_pwm_ref = 11.8;// en microsecondes
    somme_0 = somme_1 = somme_2 = somme_3 = 0.0;
    inter2 = 0;
    adc_init(4);
    timer2_init(TE_MESURES);
    reglage=1;
    delay(3000);
    reglage=0;
    init_pwm_timer1_fixe(period_pwm,0.5);
    digitalWrite(ENABLE,HIGH);
    
    ajust=1;
    temps = 0;
    f_plus = f_moins = 0;
}
                    
void loop() {
    delay(500);
    temps+= 0.5;
    if (f_plus) {
      period_pwm += 6.25e-2;
      f_plus=0;
    }
    if (f_moins) {
      period_pwm -= 6.25e-2;
      f_moins=0;
    }
    init_pwm_timer1_fixe(period_pwm,0.5);
    float R = Ueff/Ieff;
    Serial.println("t (s) = "+String(temps)+" period (us)= "+String(period_pwm)+" dephasage = "+String(dephasage)+" Ueff(V) = "+String(Ueff)+" Ieff(A) = "+String(Ieff)+ " R = "+String(R)+" T = "+String(Temp));

}
                      